
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.ctms_scenario import CtmsScenario

from eclinical.standard.steps.ctms.ctms_find_study import CtmsFindStudy


class CtmsFindStudySite(StandardStep):
    Name = "ctms_find_study_site.py"
    Info = "ctms_sites_of_study"

    def __init__(self, service, scenario: CtmsScenario):
        super().__init__(service, scenario)
        self.service.context[self.Info] = []

    def _pre_processor(self):
        CtmsFindStudy(self.service, self.scenario).run()

    def ignore(self): return self.service.context[CtmsFindStudy.Info] is None

    def call_back(self, **kwargs):
        self.service.context[self.Info].extend(kwargs.get("query").list())

    def path_variable(self): return dict(pageNo=1)

    def data(self): return dict(studyId=self.service.context[CtmsFindStudy.Info].get("id"))

    def _execute(self):
        super()._execute()
        self.service.study_management_site_list(path_variable=self.path_variable(), data=self.data())



import os.path
from typing import IO

# import cjen.dada.smile
import cjen
from cjen.dada import smile
import mitmproxy
from urllib.parse import urlparse, parse_qs, urlunsplit, urlunparse, parse_qsl

from eclinical.record.ext.api.api_spec import ApiSpec
from eclinical.record.ext.api.swagger import Swagger
from eclinical.scaffold.template.service import Service


# TODO 装饰器添加方法
# TODO 装饰器如果不存在文件，则生成文件，已存在则忽略


# @cjen.dada.smile.haha()
# def new_file():


def create_service(func):
    def __inner__(self):
        if not os.path.exists(f'{self.service_template.file_name}.py'):
            @cjen.haha(LogPath=os.path.curdir, LogName=f'{self.service_template.file_name}.py', Mode='a')
            def write(msg, io: IO):  io.write(msg)
            write(self.service_template.output_service_py())
        func(self)
        return __inner__

    return __inner__


class EclinicalService:
    def __init__(self, flow):
        self.flow = flow
        self.service_template = Service(service_name=self.name, app=self.app)

    @property
    def name(self):
        return self.flow.request.path.split("/")[2]

    @property
    def app(self):
        if self.name == "admin":
            return "portal"
        return self.name

    @property
    def method(self): return self.flow.request.method

    @property
    def api_url(self):
        # api = "/".join(self.flow.request.path.split("/")[2:])
        return ApiSpec.api(Swagger(), self.flow.request)
        # result = urlparse(api)
        # query = "&".join([f'{key}={{{key}}}' for key in parse_qs(result.query).keys()])
        # return urlunparse(["", "", result.path, result.params, query, result.fragment])

    def generate(self):
        self.append_api()

    @create_service
    def append_api(self):
        @cjen.haha(LogPath=os.path.curdir, LogName=f'{self.service_template.file_name}.py', Mode='a')
        def write(msg, io: IO):  io.write(msg)
        self.service_template.output_method(self.api_url, self.flow)
        # write(self.api_url + "\n")




import cjen
from cjen import MetaJson


class UserGroup(MetaJson):
    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.code")
    def code(self): ...
    
    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.id")
    def id(self): ...
    

class QueryUserGroups(MetaJson):
    
    @cjen.operate.common.value
    @cjen.operate.json.one(json_path="$.payload.nextPage")
    def nextPage(self): ...
    
    @cjen.operate.common.value
    @cjen.operate.json.listOf(clazz=UserGroup)
    @cjen.operate.json.one(json_path="$.payload.list")
    def user_groups(self) -> list[UserGroup]: ...

    @cjen.operate.asserts.equal(value=200)
    @cjen.operate.json.one(json_path="$.procCode")
    def procCode(self): ...



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep


class CtmsCommonCountry(StandardStep):
    Name = "ctms_common_country.py"
    Country = "CtmsCommonCountry"

    def __init__(self, service, scenario: Scenario):
        super().__init__(service, scenario)

    def ignore(self):
        try:
            if self.service.context[self.Country] is not None:
                return True
        except Exception as e:
            return False

    def call_back(self, **kwargs):
        # self.service.context[self.Country] = [{f'{country.name()}': country.id()} for country in kwargs.get("countries").list()]
        self.service.context[self.Country] = dict(zip([country.name() for country in kwargs.get("countries").list()],
                                                                                                 [country.id() for country in kwargs.get("countries").list()]))

    def _execute(self):
        super()._execute()
        self.service.common_country()



import os.path
import sys
from multiprocessing import Process
from mitmproxy.tools.main import mitmdump


class MitmProcess(Process):
    def __init__(self):
        Process.__init__(self)

    def set_host(self, host): self.host = host

    def run(self) -> None:
        # sys.argv = ['mitmdump', '-s', os.path.join(os.path.dirname(__file__), "mitm_addons.py"), '-p',
        #             '9999', '--ignore-hosts', '.*443$', '--set', 'upstream_cert=false', 'block_global=false']
        # sys.argv = ['mitmdump', '-s', os.path.join(os.path.dirname(__file__), "mitm_addons.py"), '-p',
        #             '9999', '--ignore-hosts', f'^(?![0-9\.]+:)(?!([^\.:]+\.)*{self.host}:80$', '--set',
        #             'upstream_cert=false']
        # sys.argv = ['mitmdump','-q', '-s', os.path.join(os.path.dirname(__file__), "mitm_addons.py"), '-p',
        #             '9999', '--allow-hosts', f'.*{self.host}:80$', 'block_global=false']
        sys.argv = ['mitmdump','-q', '-s', os.path.join(os.path.dirname(__file__), "mitm_addons.py"), '-p',
                    '9999', '--allow-hosts', f'.*dev-03-app-01.chengdudev.edetekapps.cn:81$', 'block_global=false']
        # argvs = [ '-s', os.path.join(os.path.dirname(__file__), "mitm_addons.py"), '-p',
        #             '9999', '--allow-hosts', '.*52.82.79.202:81$', '--set','block_global=false']
        mitmdump()

    def stop(self) -> None:
        if self.is_alive():
            self.kill()



import os

import yaml

from eclinical.execption import required, optional


def value(func):
    def __inner__(self, *args, **kwargs):
        return self.env.get(func.__name__), func.__name__

    return __inner__


class Environment:

    def __init__(self, envir, file_path):
        self.file_path = file_path
        with open(file_path, "r", encoding="utf-8") as f:
            self.env = yaml.load(f.read(), Loader=yaml.FullLoader).get("ENV").get(envir)

    @classmethod
    def loader(cls, envir):
        cur_dir = os.getcwd()
        while os.path.dirname(cur_dir) != cur_dir:
            for root, dirs, files in os.walk(cur_dir):
                for file in files:
                    if file == "environment.yaml":
                        with open(os.path.join(root, file), "r", encoding="utf-8") as f:
                            if yaml.load(f.read(), Loader=yaml.FullLoader).get("ENV").get(
                                envir) is not None: return Environment(envir=envir
                                                                       , file_path=os.path.join(root, file))
            cur_dir = os.path.dirname(cur_dir)
        raise Exception("没有发现配置文件")

    @property
    @required
    @value
    def public_key(self) -> str:
        ...

    @property
    @optional
    @value
    def uri(self) -> str:
        ...

    @property
    @optional
    @value
    def user(self) -> str:
        ...

    @property
    @optional
    @value
    def password(self) -> str:
        ...

    @property
    @optional
    @value
    def company(self) -> str:
        ...

    @property
    @optional
    @value
    def sponsor(self) -> str:
        ...

    @property
    @optional
    @value
    def study(self) -> str:
        ...

    @property
    @optional
    @value
    def life_cycle(self) -> str:
        ...

    @property
    @optional
    @value
    def db(self) -> dict:
        ...


if __name__ == '__main__':
    print(Environment.loader("US_DEMO_EDETEK_PORTAL"))




from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.hierarchies.portal_find_sponsor import PortalFindSponsor
from eclinical.standard.steps.portal.hierarchies.portal_find_study import PortalFindStudy


class PortalCreateStudy(StandardStep):
    Name = "portal_create_study"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def _pre_processor(self):
        PortalFindSponsor(self.service, self.scenario).run()

    def ignore(self):
        PortalFindStudy(self.service, self.scenario).run()
        return self.service.context[PortalFindStudy.Id] is not None

    def _execute(self):
        super()._execute()
        self.service.hierarchies_new_study(data=self.data())

    def study(self): return self.scenario.study()

    def sponsor(self): return self.scenario.sponsor()

    def data(self):
        return dict(active=True, description="", name=self.study(),
                    sponsorId=self.service.context[PortalFindSponsor.Id],
                    subjectManagement=5)



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.user_groups.portal_find_usergroup import PortalFindUserGroup


class PortalGetUserGroupInfo(StandardStep):
    Name = "portal_get_user_group_info"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def _pre_processor(self):
        PortalFindUserGroup(self.service, self.scenario).run()

    def life_cycle(self):
        return self.scenario.life_cycle()

    def call_back(self, **kwargs):
        for company_multiple_env in kwargs.get("query").company_multiple_envs():
            if company_multiple_env.name() == self.life_cycle():
                self.service.context["company_multiple_env_id"] = company_multiple_env.id()

    def _execute(self):
        super()._execute()
        self.service.usergroups_user_group_info(path_variable=self.path_variable())

    def path_variable(self):
        return dict(userGroup_id=self.service.context["user_group_id"])



from cjen import BigTangerine
from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario


class PortalGetUserRole(StandardStep):
    Name = "portal_get_user_role.py"
    Id = "portal_user_role_id"

    def __init__(self, service: BigTangerine, scenario: PortalScenario):
        super().__init__(service, scenario)

    def path_variable(self):
        return dict(PageNo=1)

    def role(self):
        return self.scenario.role()

    def _execute(self):
        super()._execute()
        self.service.userrole_api_get_user_roles(path_variable=self.path_variable())

    def call_back(self, **kwargs):
        self.service.context[self.Id] = None
        for role in kwargs.get("user_role").list():
            if role.code() == self.role():
                self.service.context[self.Id] = role.id()



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep

from eclinical.standard.scenarios.portal_scenario import PortalScenario


class PortalLifeCycle(StandardStep):
    Name = "portal_life_cycle"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def call_back(self, **kwargs):
        for env in kwargs.get("envs").payload():
            if env.name() == self.life_cycle():
                self.service.context[self.life_cycle()] = env.id()

    def life_cycle(self):
        return self.scenario.life_cycle()

    def _execute(self):
        super()._execute()
        self.service.hierarchies_get_env_list(life_cycle=self.life_cycle())



import cjen
from cjen import BigTangerine

import eclinical


class EDiaryLoginService(BigTangerine):
    @cjen.headers.basicHeaders(headers={
        "Accept": "application/json, text/plain, */*",
        "Content-Type": "application/json"
    })
    def __init__(self, environment: eclinical.Environment):
        super().__init__()
        self.environment = environment
        self.base_url = self.__dict__.get("base_url", self.environment.uri)
        self.context["userName"] = self.context.get("userName") or self.environment.user
        self.context["password"] = self.context.get("password") or self.environment.password
        self.edc_auth()
        self.edc_auth()

    @cjen.http.post_mapping(uri="admin/auth/subject/login")
    @cjen.jwt(key="Authorization", json_path="$.payload.jwtAuthenticationResponse.token")
    def auth(self, path_variable, data, resp=None, **kwargs): ...

    @cjen.http.post_mapping(uri="edc/subject/auth/login")
    @cjen.jwt(key="Authorization", json_path="$.payload.jwtAuthenticationResponse.token")
    def edc_auth(self, resp=None, **kwargs): ...



from cjen.sco.scenario import Scenario
from cjen.sco.standard_step import StandardStep


from eclinical.standard.scenarios.portal_scenario import PortalScenario


class PortalFindUserGroup(StandardStep):
    Name = "portal_find_user_group"
    Id = "user_group_id"

    def __init__(self, service, scenario: PortalScenario):
        super().__init__(service, scenario)

    def user_group(self):
        return self.scenario.user_group()

    def call_back(self, **kwargs):
        self.service.context[self.Id] = None
        for user_group in kwargs.get("query").user_groups():
            if user_group.code() == self.user_group():
                self.service.context[self.Id] = user_group.id()

    def _execute(self):
        super()._execute()
        self.service.usergroups_get_user_group(path_variable=dict(pageNo=1))



import os.path
from eclinical import Environment

from eclinical.standard.portal.portal_api import PortalApi
from eclinical.standard.scenarios.portal_scenario import PortalScenario
from eclinical.standard.steps.portal.hierarchies.portal_create_study import PortalCreateStudy
from eclinical.standard.steps.portal.hierarchies.portal_startup_study import PortalStartupStudy


class ScoCreateStudy:
    def __init__(self, scenario_dir, environment: Environment):
        self.scenario = PortalScenario(scenario_dir)
        self.service = PortalApi(environment)
        self.scenario.append_step(PortalCreateStudy, self.service)
        self.scenario.append_step(PortalStartupStudy, self.service)

    def run(self): self.scenario.run()


if __name__ == '__main__':
    sco_dir = os.path.join("new_study")
    envir = Environment.loader("DEV_01")
    sco = ScoCreateStudy(sco_dir, envir)
    sco.run()
